import numpy as np
import pylab as pl

files = """
200.2000.txt
400.2000.txt
600.2000.txt
800.2000.txt
1200.1000.txt    
1600.200.txt     
""".split()

nlines = len(open('jacobidata/' + files[0]).readlines())
nfiles = len(files)

wtimes = np.zeros((2, nfiles, nlines))
nprocs = np.zeros((2, nfiles, nlines), int)
sizes = np.zeros(nfiles)
ncpus = nprocs[0, 0, :]

for d, dname in enumerate(['jacobidata', 'redblackdata']):
    for f, fname in enumerate(files):
        size, iters, txt_ = fname.split('.')
        size = int(size)
        iters = int(iters)
        sizes[f] = size

        tokens = open(dname + '/' + fname).read().split()
        procs = map(int, tokens[::2])
        wtime = map(float, tokens[1::2])
        wtimes[d, f, :] = np.array(wtime) / iters
        nprocs[d, f, :] = procs

for i in range(nlines)[::2]:
    pl.plot(sizes, wtimes[1, :, i] / wtimes[0, :, i], 'o',
            label='cpus=%d' % ncpus[i])
pl.legend(loc=0)
pl.xlabel('Matrix size')
pl.ylabel('gauss-seidel time / jacobi time')

pl.figure()
for i in range(nlines)[::2]:
    pl.plot(np.log(sizes**.5), np.log(wtimes[0, :, i]), 'o',
            label='cpus=%d' % ncpus[i])
pl.xlabel('ln(M * N)')
pl.ylabel('ln(walltime)')
pl.legend(loc=0)

pl.show()
