  subroutine update(T, U, F, nx, ny)
    implicit none

    integer, intent(in) :: nx, ny
    real(kind=8), dimension(ny, nx) :: T, U, F
        
    integer :: i, j
!$omp do
    do j=2, nx - 1
       do i=2, ny - 1
          U(i, j) = 0.25 * (T(i-1, j) + T(i+1, j) + T(i, j-1) + T(i, j+1) &
               + F(i, j))
       enddo
    enddo
!$omp end do

!$omp do
    do j=2, nx - 1
       do i=2, ny - 1
          T(i, j) = U(i, j)
       enddo
    enddo
!$omp end do
  end subroutine update





  subroutine update_gs(T, F, nx, ny)
    implicit none

    integer :: nx, ny
    real(kind=8), dimension(ny, nx) :: T, F
        
    integer :: i, j

    do j=2, nx - 1
       do i=2, ny - 1
          T(i, j) = 0.25 * (T(i-1, j) + T(i+1, j) + T(i, j-1) + T(i, j+1) &
               + F(i, j))
       enddo
    enddo       

  end subroutine update_gs


  subroutine solve(T, U, F, nx, ny, method, maxiter, interval, maxerr)
    implicit none
    
    integer :: nx, ny, method
    real(kind=8), dimension(ny, nx) :: T, U, F
    real(kind=8) :: err, maxerr
    integer :: i, interval, maxiter
    
    if (.not.((method.eq.gs).or.(method.eq.jac))) then
       print*, 'Unknown method!'
       stop
    endif

!$omp parallel shared(T, U, F, nx, ny)
    do i=1,maxiter ! we don't want to run *that* many iterations
       !if (.false.) then
       !if (mod(i, interval).eq.0) then
       !   if (method.eq.gaussseidel) then
       !      call update_gs_and_calculate_norm(T, F, nx, ny, err)
       !   else
       !      call update_and_calculate_norm(T, U, F, nx, ny, err)
       !      T = U
       !   endif
       !   if(err.le.maxerr) then
       !      print*, 'Converged after', i, 'iterations'
       !      return
       !   endif
       !else
       if (method.eq.gs) then
          call update_gs(T, F, nx, ny)
       elseif (method.eq.jac) then
          call update(T, U, F, nx, ny)
       endif
       !endif
    enddo
!$omp end parallel
    !print*, 'DID NOT CONVERGE IN MANY ITERATIONS'
  end subroutine solve

!!$
!!$  subroutine temporally_blocking_gaussseidel(T, F, nx, ny, maxiter, interval, &
!!$       maxerr)
!!$    implicit none
!!$    
!!$    integer, intent(in) :: nx, ny, interval, maxiter
!!$    real(kind=8), dimension(ny, nx), intent(inout) :: T
!!$    real(kind=8), dimension(ny, nx), intent(in) :: F
!!$    real(kind=8), intent(in) :: maxerr
!!$
!!$    real(kind=8) :: err
!!$    integer :: n, i, j
!!$    
!!$    integer :: proc, iprocs, istart, istop, iblock, iblocks, iblocksize
!!$    integer :: jstart, jstop, jblocksize, jblocks
!!$    
!!$    nprocs = omp_get_max_threads()
!!$
!!$    ! Make sure problem is perfectly quadratic
!!$    if (nx.ne.ny) then
!!$       print*, 'only square matrices please'
!!$       stop
!!$    endif
!!$    if (mod(nx, nprocs).ne.0) then
!!$       print*, 'bad CPU count'
!!$       stop
!!$    endif
!!$    jblocks = iprocs
!!$    jblocksize = nx / nprocs
!!$    iblocksize = jblocksize
!!$
!!!!omp parallel shared(T, F, nx, ny, maxiter, interval, maxerr, &
!!!!omp   & iprocs, iblocks, iblocksize, jblocksize, jblocks) &
!!!!omp   & private(proc, n, i, j, istart, istop, jstart, jstop)
!!$    !proc = omp_get_thread_num()
!!$    !jstart = 1 + nx * proc / nprocs
!!$    !jstop = jstart + nj
!!$
!!$    !do n=1,maxiter
!!$       !do jblock=1, nblocks
!!$       !   istart = 1 + ny * jblock / nblocks
!!$       !   istop = istart + ni
!!$          
!!$       !do j = jstart, jend!do j=2, nx - 1
!!$       !   do i=istart, istop
!!$       !      T(i, j) = 0.25 * (T(i-1, j) + T(i+1, j) + T(i, j-1) + T(i, j+1) &
!!$       !           + F(i, j))
!!$       !   enddo
!!$       !enddo
!!$    !enddo
!!!!omp end parallel
!!$  end subroutine temporally_blocking_gaussseidel
