clear all;
close all;
clc;

load 'field.dat'

x = field(:,1);
y = field(:,2);
t = field(:,3);

X = reshape(x,sqrt(length(field)),sqrt(length(field)));
Y = reshape(y,sqrt(length(field)),sqrt(length(field)));
T = reshape(t,sqrt(length(field)),sqrt(length(field)));

% Jacobi field
jMsiz = [64   256   1024  4096  16384 65536   262144];
jnint = [179  802   3292  13029 50631 194913  745795];
jtime = [0.03 0.027 0.088 0.848 16.83 264.39  4142.29];
jipes = jnint./jtime;
[a]= polyfit(log(jMsiz),log(jnint),1);
Yline = exp(a(1)*log(jMsiz)+a(2));
str1 = sprintf('Jacobi slope= %.2f',a(1));

% Gauss Seidel field
gMsiz = [64   256   1024  4096  16384 65536  262144];
gnint = [93   417   1712  6789  26439 102003  391195];
gtime = [0.02 0.029 0.07  0.807 15.02 244.26 4041.69];
gipes = gnint./gtime;
[b]= polyfit(log(gMsiz),log(gnint),1);
Gline = exp(b(1)*log(gMsiz)+b(2));
str2 = sprintf('GS slope= %.2f',a(1));

figure(11)
surf(X,Y,T)
xlabel('x','Interpreter','latex','FontSize',20)
ylabel('y','Interpreter','latex','FontSize',20)
zlabel('T','Interpreter','latex','FontSize',20)
%print -depsc tfield.eps


figure(1)
loglog(jMsiz,jnint,'o',jMsiz,Yline,gMsiz,gnint,'*',gMsiz,Gline,'-.')
xlabel('Number of Elements $N^2$','interpreter','latex','FontSize',20)
ylabel('Number of Interations $\#$','interpreter','latex','FontSize',20)
set(legend('Jacobi',str1,'GS',str2,'Location','NW'),'Interpreter','latex','FontSize',16)
legend boxoff
print -deps q1a.eps

figure(2)
loglog(jMsiz,jtime,'o',gMsiz,gtime,'s')
xlabel('Number of Elements $N^2$','interpreter','latex','FontSize',20)
ylabel('Wall clock time $[s]$','interpreter','latex','FontSize',20)
set(legend('Jacobi','Gauss Seidel','Location','NW'),'Interpreter','latex','FontSize',16)
legend boxoff
print -deps q1b.eps

figure(3)
semilogx(jMsiz,jipes,'o',gMsiz,gipes,'s')
xlabel('Number of Elements $N^2$','interpreter','latex','FontSize',20)
ylabel('Number of Int. per second $[int/s]$','interpreter','latex','FontSize',20)
set(legend('Jacobi','Gauss Seidel','Location','NE'),'Interpreter','latex','FontSize',16)
%legend boxoff
print -deps q1c.eps